/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef NPP_DPPHISTOGRAM_H
#define NPP_DPPHISTOGRAM_H

#include "DPPLiveProcess.h"
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/thread.hpp>
#include <stdint.h>

namespace npp {

class DPPHistogram : public DPPLiveProcess {

public:
	//! Type of controller
	static const std::string TYPE;

	DPPHistogram(const std::string& name);
	DPPHistogram(const DPPHistogram& controller);
	virtual ~DPPHistogram();

	virtual void postConfiguration();

	virtual void start();
	virtual void stop();
	virtual void raz();

private:
	static const std::string LIVEHISTO;
	static const char SHARED_MEMORY_NAME[];
	static const char SHARED_MEMORY_DATA_NAME[];
	static const char SHARED_MEMORY_ADCRATES_NAME[];
	static const std::string HISTOGRAM;
	static const std::string DATA_PATH;

	void resetData();
	void startLiveHisto();
	void stopLiveHisto();

	void executeLiveHisto();
	void executeRefreshHisto();

	Property<int32> triggerChannel;
	Property<int32> delay;
	Property<int32> dT;
	Property<int32> maxBlock;
	Property<int32> blockSize;
	Property<bool> test;
	Property<std::string> testFileName;
	ArrayProperty<int32> xData;
	Property<int32> one;
	Property<int32> refreshTimeS;
	DynamicArrayProperty<int32> histogram;
	DynamicProperty<int32> ADCId;
	DynamicProperty<float64> ADCRate;

	int32 * _xData;
	float64 * _ADCRates;
	boost::interprocess::managed_shared_memory * _segment;

	boost::mutex _mutex;
	boost::thread * _liveCoincidenceThread;

	bool _liveRunning;
	boost::thread * _liveRefreshThread;

	static const int32 SEGMENT_SIZE = 65536 * 128 * sizeof(int32_t);
	static const int32 MAX_ADC = 128;
};

}

#endif
